//------------------------------------
//  (c) Reliable Software, 1996, 97
//------------------------------------

//
// Plane
//
void PlaneSet(Plane *P,Point *center,Vector *perpendicular,OpticalProp *pr,COLORREF *pixels,int width,int height)
{
  Vector Temp = {0,0,1};

  P->Obj.Type = OBJECT_PLANE;
  P->Obj.OpticalProperty = *pr;

  P->Center = *center;
  P->Pixels = pixels;
  P->Width  = width;
  P->Height = height;

  P->Normal = *perpendicular;
  NormalizeVector(&P->Normal);

  P->Cos_inv = 1.0 / GetDotProductOfVectors(&P->Normal, &Temp);
}

// Vector normal at intersection
Vector PlaneNormal(Plane *P,Point *pt)
{
  return P->Normal;
}

void PlaneGetPixel(Plane *P,Point *pt, Color *color)
{
  int x,y;
  COLORREF rgb;

	// texture mapping
	// convert to local plane coordinates
	x = (int)ceil (pt->x * P->Cos_inv);
	y = (int)ceil (pt->y);
	// divide them modulo tile size
	x %= P->Width;
	if (x < 0) x += P->Width;
	y %= P->Height;
	if (y < 0) y += P->Height;
	// find the corresponding pixel in the tile
	rgb = P->Pixels [y * P->Width + x];
	SetColorFromRGB(color, rgb);
}

bool PlaneIntersect(Plane *P,Ray* pRay)
{
  double inc,t;
  Vector oc;
  // ( org + t * dir - center ) * normal = 0

  // cosine of incidence angle
  inc = GetDotProductOfVectors(&pRay->Direction, &P->Normal);
  if ( inc >= EPSILON )
	  return false;

  SetVectorFromPoints(&oc, &P->Center, &pRay->Origin);

  t = GetDotProductOfVectors(&oc, &P->Normal);
  t /= inc;
  if ( t > EPSILON )
  {
    RayRecord(pRay, t, (VisibleObject*)P);
    return true;
  }
  return false;
}

Shader *PlaneCreateShader(Plane *P,Ray *pRay)
{
  Shader *pShader;

  pShader = (Shader*)malloc(sizeof(Shader));
  ShaderSet(pShader, SHADER_PLANE, pRay, (VisibleObject*)P);
  return pShader;
}


