//------------------------------------
//  (c) Reliable Software, 1996, 97
//------------------------------------
void RaySet(Ray *R,RayType RT,Point *Org,Vector *Dir,int Depth,double Refr)
{
  R->Type = RT;

  R->Origin = *Org;
  R->Direction = *Dir;

  NormalizeVector(&R->Direction);
  R->Depth = Depth;

  R->Obj = NULL;
  R->Dist = HUGE_DIST;

  R->Refr = Refr;         // Only for refracted rays
  R->Attenuation = 1.0;   // Only for shadow rays
}

void RayHit(Ray *R,Color *c)
{
  int i;

  for (i = 0; i < ObjCount; i++)
  {
     ObjectIntersect(Objects[i], R);
  }
  RayDoShade(R, c);
}

// The intersection point = direction * dist + origin
Point RayIntersect(Ray *R)
{
  Point intersect;
  Vector tmp;

  tmp = R->Direction;
  MultiplyVector(&tmp, R->Dist);
  intersect = R->Origin;
  AddVectorToPoint(&intersect, &tmp);
  return intersect;
}

void RayRecord(Ray *R,double dist, VisibleObject *obj)
{
  if (R->Type == rtShadow)
  {
    if (dist < R->Dist && dist > 0.0)
    {
      if (obj->OpticalProperty.Transparency) // Pass through but record attenuation
      {
        R->Attenuation *= obj->OpticalProperty.Transparency;
      }
      else
      {
        R->Dist = dist;
        R->Obj  = obj;
      }
    }
  }
  else
  {
    if (dist < R->Dist && dist > EPSILON)
    {
      R->Dist = dist;
      R->Obj  = obj;
    }
  }
}

// Shading
void RayDoShade(Ray *R,Color *c)
{

  if (R->Type == rtShadow)
  {
    if (R->Obj != NULL && R->Obj->OpticalProperty.IsLuminous)
    {
      *c = R->Obj->OpticalProperty.Col;
    }
    else
    {
      SetColor(c, 0, 0, 0);
    }
  }else{
    Shader *pShader;

    if (R->Obj == NULL || R->Depth>MaxDepth)
    {
      SetColor(c, 0, 0, 0);
      return;
    }

    if (R->Obj->OpticalProperty.IsLuminous)
    {
      *c = R->Obj->OpticalProperty.Col;
      return;
    }


    // RECURSION
    pShader = (Shader*)ObjectCreateShader((VisibleObject*)R->Obj, R);
    ShaderGetColor(pShader, c);
    ShaderFree(pShader);
    free(pShader);
  }
}

